/**
 *  Copyright 2011 Carsten Gräf
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 */

package de.twenty11.skysail.common.filters;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

/**
 * An AndFilter is a filter with FilterType "AND" with only one level of
 * subfilters
 * 
 * The value can be a list of further filters.
 * 
 * @author carsten
 * 
 */
public class AndFilter extends AbstractFilter {

    /**
     * 
     */
    private static final long serialVersionUID = 1608849901671774650L;

    /**
     * Create an AndFilter object from a Map of String to String. Please note
     * that all filter entries will automatically be "AND"ed and the subfilter
     * are of type "Substring".
     * 
     * @return a new Filter instance
     */
    public static Filter newInstance(Map<String, String> valuesMap) {
        return newInstance(valuesMap, valuesMap.keySet());
    }

    public static Filter newInstance(Map<String, String> valuesMap, Set<String> validKeys) {
        if (valuesMap == null || valuesMap.size() == 0) {
            return new AndFilter(FilterType.EMPTY);
        }
        List<Filter> operands = new ArrayList<Filter>();
        for (Entry<String, String> entry : valuesMap.entrySet()) {
            if (validKeys.contains(entry.getKey())) {
                // Create one filter for each key with non-empty value
                String value = valuesMap.get(entry.getKey());
                if (value != null) {
                    Filter filter = new SubstringFilter(entry.getKey(), entry.getValue());
                    operands.add(filter);
                }
            }
        }
        return new AndFilter(FilterType.AND, null, operands);
    }


    /**
     * 
     * @param type
     */
    // TODO ugly
    public AndFilter(FilterType type) {
        setFilterType(type);
        setAttribute(null);
        setSubfilters(null);
        setValue(null);
    }

    // TODO make private? new Instance?
    public AndFilter(FilterType type, String attribute, List<Filter> subfilters) {
        setFilterType(type);
        setAttribute(attribute);
        setSubfilters(subfilters);
        setValue(null);
    }

    @Override
    public String getValue(String attribute) {
        if (getSubfilters() == null || getSubfilters().size() == 0) {
            return "";
        }
        for (Filter filter : getSubfilters()) {
            if (filter.getValue(attribute) != null) {
                return filter.getValue(attribute);
            }
        }
        return "";
    }

    @Override
    public boolean match(Map<String, String> properties) {
        if (getSubfilters() == null) {
            return true;
        }
        for (Filter filter : getSubfilters()) {
            if (!filter.match(properties)) {
                return false;
            }
        }
        return true;
    }

    private String print() {
        if (getSubfilters() == null) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        sb.append("(&");
        for (Filter filter : getSubfilters()) {
            sb.append(filter.toString());
        }
        sb.append(')');
        return sb.toString();
    }

    @Override
    public String toString() {
        return print();
    }


}
